#!/bin/sh /usr/share/centrifydc/perl/run
#
# Copyright (C) 2004-2020 Centrify Corporation. All rights reserved.
#
# Machine mapper script that enforces screen locking for Ubuntu 12.04 and later.
#
# This script creates an autostart file and an autostart script to configure
# user's Gnome3 screenlock setting at user login.
#
#   This script uses the following GP:
#
#     Computer Configuration
#     -> Centrify Settings
#        -> Linux Settings
#           -> Enforce screen locking
#
#
#     timeout:
#       /org/gnome/settings-daemon/plugins/power/sleep-display-battery
#       /org/gnome/settings-daemon/plugins/power/sleep-display-ac
#       /org/gnome/desktop/session/idle-delay
#     lock:
#       /org/gnome/desktop/screensaver/lock-enabled
#       /org/gnome/desktop/screensaver/ubuntu-lock-on-suspend
#
#
#  Map:     configure user screen lock setting
#             Not Configured: reset default setting
#             Enable:         set screen lock setting
#             Disable:        reset default setting
#
#  Unmap:   reset default setting
#
#
# Parameters: <map|unmap> username mode
#   map|unmap   action to take
#   username    username
#   mode        mode (not used)
#
# Exit value:
#   0   Normal
#   1   Error
#   2   usage
#

use strict;
use lib '/usr/share/centrifydc/perl';
use CentrifyDC::Include;

use CentrifyDC::GP::Args;
use CentrifyDC::GP::General qw(:debug IsEmpty RunCommand WriteFile);
use CentrifyDC::GP::Conf qw(GetAllSettingsFromConfFile);
use CentrifyDC::GP::RegHelper;

my $REGKEY = 'software/policies/centrify/unixsettings/screenlock';
my $REGVAL_LOCK = 'ForceScreenLocking';
my $REGVAL_TIMEOUT = 'LockTimeOut';

my $FILE_AUTOSTART = '/etc/xdg/autostart/cdc-set-screenlock.desktop';
my $FILE_SCRIPT = '/usr/local/bin/cdc-set-screenlock';

my $DCONF      = '/usr/bin/dconf';
my $GSETTINGS  = '/usr/bin/gsettings';

my %DCONF_KEYS = (
    'sleep-display-battery'  => '/org/gnome/settings-daemon/plugins/power/sleep-display-battery',
    'sleep-display-ac'       => '/org/gnome/settings-daemon/plugins/power/sleep-display-ac',
    'idle-delay'             => '/org/gnome/desktop/session/idle-delay',
    'lock-enabled'           => '/org/gnome/desktop/screensaver/lock-enabled',
    'ubuntu-lock-on-suspend' => '/org/gnome/desktop/screensaver/ubuntu-lock-on-suspend',
);

my %GSETTINGS_KEYS = (
    'sleep-display-battery'  => 'org.gnome.settings-daemon.plugins.power',
    'sleep-display-ac'       => 'org.gnome.settings-daemon.plugins.power',
    'idle-delay'             => 'org.gnome.desktop.session',
    'lock-enabled'           => 'org.gnome.desktop.screensaver',
    'ubuntu-lock-on-suspend' => 'org.gnome.desktop.screensaver',
);

# hash reference of generated gsettings settings
my $g_settings = {};

sub IsSupportedUbuntu();
sub GenerateSettings();

sub CreateAutostartFiles();
sub RemoveAutostartFiles();
sub SetFilePerm($$$$);

sub Map();
sub UnMap();



# >>> SUB >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

#
# check if current OS is supported.
# this script supports Ubuntu 12.04 and later.
#
#   return: 1       - yes
#           0       - no
#           undef   - failed
#
sub IsSupportedUbuntu()
{
    my $FILE_LSB_RELEASE = '/etc/lsb-release';

    if (! -e $FILE_LSB_RELEASE)
    {
        return 0;
    }

    my $settings = GetAllSettingsFromConfFile($FILE_LSB_RELEASE);
    defined($settings) or return undef;

    my $distrib_id = $settings->{'DISTRIB_ID'};
    defined($distrib_id) and $distrib_id =~ s/\s*$//;

    my $distrib_release = $settings->{'DISTRIB_RELEASE'};
    defined($distrib_release) and $distrib_release =~ s/\s*$//;

    DEBUG_OUT("DISTRIB_ID: [$distrib_id]  DISTRIB_RELEASE: [$distrib_release]");

    if ($distrib_id ne 'Ubuntu')
    {
        return 0;
    }

    if ($distrib_release < 12.04)
    {
        return 0;
    }

    return 1;
}

#
# generate settings.
# need to translate registry setting to the gsettings setting.
#
#   return: 1       - successful
#           undef   - failed
#
sub GenerateSettings()
{
    my $reg_lock = CentrifyDC::GP::RegHelper->new('map', 'machine', $REGKEY, $REGVAL_LOCK, undef);
    $reg_lock or return undef;
    $reg_lock->load();

    my $reg_timeout = CentrifyDC::GP::RegHelper->new('map', 'machine', $REGKEY, $REGVAL_TIMEOUT, undef);
    $reg_timeout or return undef;
    $reg_timeout->load();

    my $lock = $reg_lock->get('current');
    if (defined($lock))
    {
        $lock = ($lock eq 'yes') ? 'true' : 'false';
        $g_settings->{'lock-enabled'} = $lock;
        $g_settings->{'ubuntu-lock-on-suspend'} = $lock;
    }

    my $lock_timeout = $reg_timeout->get('current');

    if (defined($lock_timeout))
    {
        $lock_timeout = $lock_timeout * 60;

        $g_settings->{'sleep-display-battery'} = $lock_timeout;
        $g_settings->{'sleep-display-ac'} = $lock_timeout;
        $g_settings->{'idle-delay'} = $lock_timeout;
    }

    return 1;
}

#
# create autostart files.
#
#   return: 1       - successful
#           undef   - failed
#
sub CreateAutostartFiles()
{
    DEBUG_OUT("Create autostart files");

    umask 0022;

    my $str_autostart = "[Desktop Entry]
Name=Set screenlock
Exec=/usr/local/bin/cdc-set-screenlock
Type=Application
NoDisplay=true
X-Ubuntu-Gettext-Domain=cdc-set-screenlock
X-GNOME-AutoRestart=false
OnlyShowIn=GNOME;Unity;
";        

    if (! WriteFile($FILE_AUTOSTART, $str_autostart))
    {
        ERROR_OUT("Cannot write $FILE_AUTOSTART");
        return undef;
    }

    my $str_script = "#!/bin/sh

logger '=== set screenlock time to $g_settings->{'sleep-display-battery'}. lock: $g_settings->{'lock-enabled'} ==='

/usr/bin/gsettings set 'org.gnome.settings-daemon.plugins.power' 'sleep-display-battery' $g_settings->{'sleep-display-battery'}
/usr/bin/gsettings set 'org.gnome.settings-daemon.plugins.power' 'sleep-display-ac' $g_settings->{'sleep-display-ac'}
/usr/bin/gsettings set 'org.gnome.desktop.session' 'idle-delay' $g_settings->{'idle-delay'}
/usr/bin/gsettings set 'org.gnome.desktop.screensaver' 'lock-enabled' $g_settings->{'lock-enabled'}
/usr/bin/gsettings set 'org.gnome.desktop.screensaver' 'ubuntu-lock-on-suspend' $g_settings->{'ubuntu-lock-on-suspend'}
";

    if (! WriteFile($FILE_SCRIPT, $str_script))
    {
        ERROR_OUT("Cannot write $FILE_SCRIPT");
        return undef;
    }

    SetFilePerm($FILE_AUTOSTART, 0644, 0, 0) or return undef;
    SetFilePerm($FILE_SCRIPT, 0755, 0, 0) or return undef;

    return 1;
}

#
# remove autostart files.
#
#   return: 1       - successful
#           undef   - failed
#
sub RemoveAutostartFiles()
{
    DEBUG_OUT("Remove autostart files");

    if (-e $FILE_SCRIPT)
    {
        DEBUG_OUT("Remove $FILE_SCRIPT");
        if (! unlink($FILE_SCRIPT))
        {
            ERROR_OUT("Cannot remove $FILE_SCRIPT");
            return undef;
        }
    }

    if (-e $FILE_AUTOSTART)
    {
        DEBUG_OUT("Remove $FILE_AUTOSTART");
        if (! unlink($FILE_AUTOSTART))
        {
            ERROR_OUT("Cannot remove $FILE_AUTOSTART");
            return undef;
        }
    }

    return 1;
}

# set file permission and ownership
#
#   $_[0]:  file
#   $_[1]:  permission
#   $_[2]:  uid
#   $_[3]:  gid
#
#   return: 1       - successful
#           undef   - failed

sub SetFilePerm($$$$)
{
    my $file = $_[0];
    my $dest_perm = $_[1];
    my $dest_uid  = $_[2];
    my $dest_gid  = $_[3];

    defined($file) or return undef;
    defined($dest_perm) or return undef;
    defined($dest_uid) or return undef;
    defined($dest_gid) or return undef;

    if (! -e $file)
    {
        ERROR_OUT("$file not exist");
        return undef;
    }

    my $ret = 1;

    my ($perm, $uid, $gid) = (stat $file)[2, 4, 5];
    $perm = $perm & 07777;

    my $rc;
    if ($perm != $dest_perm)
    {
        DEBUG_OUT("Change $file permission from [%04o] to [%04o]", $perm, $dest_perm);
        $rc = chmod($dest_perm, $file);
        if (! $rc)
        {
            ERROR_OUT("Cannot change $file permission from [%04o] to [%04o]", $perm, $dest_perm);
            $ret = undef;
        }
    }

    if ($uid != $dest_uid || $gid != $dest_gid)
    {
        DEBUG_OUT("Change $FILE_AUTOSTART owner from $uid:$gid to $dest_uid:$dest_gid");
        $rc = chown($dest_uid, $dest_gid, $file);
        if (! $rc)
        {
            ERROR_OUT("Cannot change $file owner from $uid:$gid to $dest_uid:$dest_gid");
            $ret = undef;
        }
    }

    return 1;
}

#
# map
#
#   return: 1       - successful
#           undef   - failed
#
sub Map()
{
    my $ret = 1;

    if (! IsEmpty($g_settings->{'lock-enabled'}))
    {
        $ret = CreateAutostartFiles();
    }
    else
    {
        $ret = RemoveAutostartFiles();
    }

    return $ret;
}

#
# unmap
#
#   return: 1       - successful
#           undef   - failed
#
sub UnMap()
{
    my $ret = 1;

    $ret = RemoveAutostartFiles();

    return $ret;
}


# >>> MAIN >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

if (! IsSupportedUbuntu())
{
    DEBUG_OUT("This mapper is for Ubuntu 12.04 and later. Skip.");
    exit(0);
}

(-x $GSETTINGS) or FATAL_OUT("Cannot run $GSETTINGS");

my $args = CentrifyDC::GP::Args->new('machine');

CentrifyDC::GP::Registry::Load(undef);

if (! GenerateSettings())
{
    ERROR_OUT("Failed to generate settings.");
    exit(1);
}

my $ret = 0;

$args->isMap() ? ($ret = Map()) : ($ret = UnMap());

$ret or FATAL_OUT();


