#!/bin/sh /usr/share/centrifydc/perl/run

##############################################################################
#
# Copyright (C) 2014-2020 Centrify Corporation. All rights reserved.
#
# a Perl package supplying some basic functions to modify pam.d files
# The functions here are used by other /cli/confgurefiles perl scripts
#
##############################################################################

package autoedit_pam_d;

BEGIN
{
    use lib "/usr/share/centrifydc/scripts";
    use lib "/usr/share/centrifydc/perl";
}

use strict;
use warnings;
use File::Copy qw(copy);

use autoedit_common_utils qw(insert_cfg_lines);
use autoedit_file_lock;
use CentrifyDC::GP::General qw(:debug :system);
use CentrifyDC::Include;
use CentrifyDC::Platform qw(GetPlatform OS ARCH);

use base 'Exporter';
our @EXPORT_OK = qw(process_pam_d_files process_pam_d_file);

# process pam.d file
#
# get the content of pam.d file
# add our centriftdc code
# write all code into a temp file
# rename and replace the original pam.d file
#
# $cfgName          the pam.d file name
# $skip_stack_libs  the stack files, if meet any one, it must be a system-auth 
#                   or common-auth style pam file. Then we need to do nothing.
# other arguments   please see autoedit_common_utils.pm::insert_cfg_lines
# return            undef if anything wrong
#                   1 if succeed
sub process_pam_d_file
{
    my ($cfgName, $newLines, $libs_excluded, $newLines2, $cdc_pam_lib, 
        $cdc_long_comment, $skip_stack_libs, $dzdo_files) = @_;
    my $cfgHandler;
    my $tmpName;
    my $tmpHandler;

    my $flock = autoedit_file_lock::new("$cfgName");
    if (!defined $flock)
    {
        DEBUG_OUT("cannot lock file $cfgName");
        return undef;
    }

    $cfgName = TraverseSymLink($cfgName);
    if (!defined $cfgName)
    {
        ERROR_OUT("meet error when check symlink $cfgName");
        return undef;
    }

    # to make logic simple, read all lines here
    my $cfgLines = ReadFile("$cfgName");
    if (!defined $cfgLines)
    {
        ERROR_OUT("cannot read $cfgName");
        return undef;
    }
    my $oldLines = $cfgLines;
    my @cfgLines = split "\n", $cfgLines;
    @cfgLines = grep {$_ .= "\n"} @cfgLines;

    # Catch the dzdo files here and fix them for SUSE 12.
	# The dzdo_file array is only defined in the function arguments if
	# this is an "extra" file. Non "extra" files will thus skip this section.
	if (defined $dzdo_files)
	{
		my $platform = GetPlatform();
		
		# is this a centrify dzdo file and is this sles 12
		# and is pam_apparmor not on the system?
		if (( grep{$_ eq $cfgName} @$dzdo_files) &&
			($platform->getOS() eq OS->{SUSE}) &&
			($platform->getOSRevMajor() eq "12") &&
			( ! -f "/lib64/security/pam_apparmor.so"))
		{
			#take pam_apparmor.so out of the dzdo file
			my ($index) = grep { $cfgLines[$_] =~ /pam_apparmor.so/ } 0..$#cfgLines;
			if (defined $index)
			{
				splice(@cfgLines, $index, 1);
				$cfgLines = join "", @cfgLines;
				
				if (!WriteFile($cfgName, $cfgLines))
				{
					ERROR_OUT("Cannot write $cfgName");
					return undef;
				}
				return 1;
			}
		}
	}
	
	# return 1, if found centrify pam lib
    # it must have been updated.
    if (grep /[^#]*\b$cdc_pam_lib\b/, @cfgLines)
    {
        return 1;
    }

    # return 1, if it is a system-auth or 
    # common-auth style file
    # no need to add centrifydc code here
    if (defined $skip_stack_libs)
    {
        foreach my $lib (@$skip_stack_libs)
        {
            if (grep /^[^#]*\b$lib\s/, @cfgLines)
            {
                return 1;
            }
        }
    }

    # analyse Centrifydc code
    # put them into hash by different module name
    my %hash = ();
    my %hash2 = ();
    foreach my $module ("auth", "account", "session", "password")
    {
        my @lines  = grep /\b$module\s/, @$newLines;
        my @lines2 = grep /\b$module\s/, @$newLines2;
        $hash{$module}  = scalar(@lines)  ? [@lines]  : undef;
        $hash2{$module} = scalar(@lines2) ? [$cdc_long_comment, @lines2] : undef;
    }

    # Add our code, do nothing if meet $libs_excluded.
    # If our code is added, the added lines will be returned. Group those
    # lines together and put them to the top.
    my @topLines = ();
    my @processLines = @cfgLines;
    foreach my $module ("auth", "account", "session", "password")
    {
        if (grep /^\s*$module\s/, @processLines)
        {
            push(@topLines,
                 insert_cfg_lines(\@processLines, $hash{$module}, undef,
                                  $module, $libs_excluded->{$module}, undef));
        }
    }

    # Add the lines to the top and add comment.
    if (scalar(@topLines))
    {
        @cfgLines = ($cdc_long_comment, @topLines, @cfgLines);
    }

    # not all pam libs like our code to be above them
    # add our code behind them
    foreach my $module ("auth", "account", "session", "password")
    {
        if (grep /^\s*$module\s/, @cfgLines)
        {
            insert_cfg_lines(\@cfgLines, undef, undef,
                $module, $libs_excluded->{$module}, $hash2{$module});
        }
    }

    $cfgLines = join "", @cfgLines;

    if ($oldLines ne $cfgLines )
    {
        if (! copy($cfgName, "$cfgName.pre_cdc"))
        {
            ERROR_OUT("Cannot backup $cfgName: $!");
            return undef;
        }

        if (!WriteFile($cfgName, $cfgLines))
        {
            ERROR_OUT("Cannot write $cfgName");
            return undef;
        }
    }

    return 1;
}

# process multiple pam.d files
#
# $cfg_files        a Ref of array of the pam.d files name
# $skip_stack_libs  the stack files, if meet any one, it must be a system-auth 
#                   or common-auth style pam file. Then we need to do nothing.
# other arguments   please see autoedit_common_utils.pm::insert_cfg_lines
# return    undef if anything wrong
#           0 if have no file be updated
#           1 if succeed
sub process_pam_d_files
{
    my ($cfg_files, $cdc_pam_lib, $libs_excluded, $cfg_dir,
        $cdc_long_comment, $newLines, $newLines2, $skip_stack_libs) = @_;

    my $ret = 1;
    my $count = 0;
    foreach my $cfg_file (@$cfg_files)
    {
        if (-e $cfg_file)
        {
            my $rc = process_pam_d_file($cfg_file, $newLines, $libs_excluded,
                        $newLines2, $cdc_pam_lib, $cdc_long_comment, 
                        $skip_stack_libs);
            if (!$rc)
            {
                # meet error,
                # continue to edit other files.
                $ret = undef;
            }
            $count++;
        }
    }

    if ($count == 0)
    {
        return 0;
    }
    else
    {
        return $ret;
    }
}

1;
